# -*- coding: utf-8 -*-

from nfe.pysped.nfe import ProcessadorNFe, DANFE
from nfe.pysped.nfe.manual_401 import *
from nfe.pysped.nfe.manual_500 import *
from os.path import abspath, dirname
from OpenSSL import crypto
from StringIO import StringIO
from xml.dom import minidom
from nfe.pysped.nfe.danfe.danferetrato import *
from nfe.pysped.xml_sped.certificado import Certificado
import pytz
from datetime import datetime
local_tz = pytz.timezone('America/Sao_Paulo')
FILE_DIR = abspath(dirname(__file__))


class nf_e(object):
    """
    Especialização da biblioteca pysped.
    @author: Marcilene Ribeiro
    """

    @staticmethod
    def extrair_certificado_a1(arquivo, senha):
        """
        Extrai o conteúdo do certificado A1

        :param arquivo: binario do certificado
        :param senha: senha do certificado.
        :return: dicionário com a string do certificado, chave privada, emissor, proprietario,
        data_inicio_validade e data_final_validade.
        """
        pkcs12 = crypto.load_pkcs12(arquivo, senha)
        key = crypto.dump_privatekey(crypto.FILETYPE_PEM, pkcs12.get_privatekey())
        cert = crypto.dump_certificate(crypto.FILETYPE_PEM, pkcs12.get_certificate())
        certificado = Certificado()
        certificado.prepara_certificado_txt(cert)

        return dict(cert=cert, key=key, emissor=certificado.emissor.get('OU'),
                    proprietario=certificado.proprietario.get('CN'),
                    data_inicio_validade=certificado.data_inicio_validade,
                    data_final_validade=certificado.data_fim_validade)

    @staticmethod
    def processar_notas(xml, cert, key, ambiente, estado, contingencia):
        """
        Transmite e processa as notas fiscais eletronicas.

        :param xml: lista de xml das notas fiscais
        :param cert: certificado digital A1
        :param key: chave privada do certificado digital
        :param ambiente: ambiente da consulta. 1=Producao e 2=Homologacao,
        :param estado: UF em que sera realizado o processamento
        :param contingencia: habilita o servico de contigencia
        :return: processo e arquivos de retorno.

        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        p = ProcessadorNFe()
        p.ambiente = ambiente
        p.estado = estado
        p.versao = u'3.10'
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = True
        p.contingencia = contingencia
        p.caminho = u''
        lista_nfe = list()
        for xml in xml if isinstance(xml, list) else [xml]:
            nfe = NFe_310()
            nfe.infNFe.xml = xml
            lista_nfe.append(nfe)

        return p.processar_notas(lista_nfe)

    @ staticmethod
    def consultar_recibo(lote, cert, key, ambiente, estado, contingencia=False):
        """
        Realiza a consulta do lote de processamento de notas fiscais
        :param lote: o numero do lote (recibo) para a consulta
        :param cert: certificado digital A1
        :param key: chave privada do certificado digital
        :param ambiente: ambiente da consulta. 1=Producao e 2=Homologacao,
        :param estado: UF em que sera realizado o processamento
        :param contingencia: habilita o servico de contigencia
        :type lote: int
        :type cert: str
        :type key: str
        :type ambiente: int 
        :type estado: str
        :type contingencia: str
        :return: processo e arquivos com os dados das notas ficasis contidas no lote 
        :rtype: tuple
        
        :author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """
        p = ProcessadorNFe()
        p.versao = u'3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.contingencia = contingencia
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False

        return p.consultar_recibo(lote)

    @staticmethod
    def cancelar_nota(chave, protocolo, justificativa, cert, key, ambiente, estado,
                      contingencia=False):
        """
        Realiza o cancelamento da nfe.
        @param chave:chave da nfe
        @param protocolo: protocolo do processamento da nfe
        @param justificativa: justificativa do cancelamento
        :param cert: certificado digital A1
        :param key: chave privada do certificado digital
        :param ambiente: ambiente da consulta. 1=Producao e 2=Homologacao,
        :param estado: UF em que sera realizado o processamento
        :param contingencia: habilita o servico de contigencia
        @return: Dicionário com o envio,resposta e reason.
        """
        p = ProcessadorNFe()
        p.versao = u'3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.contingencia = contingencia
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False

        return p.cancelar_nota(chave, protocolo, justificativa)

    @staticmethod
    def inutilizar_nota(chave, justificativa, cert, key, ambiente, estado, contingencia=False):
        """
        Realiza a inutilização do número de uma nota fiscal
        @param cnpj:cnpj do emitente
        @param justificativa: justificativa da inutilização
        :param cert: certificado digital A1
        :param key: chave privada do certificado digital
        :param ambiente: ambiente da consulta. 1=Producao e 2=Homologacao,
        :param estado: UF em que sera realizado o processamento
        :param contingencia: habilita o servico de contigencia
        @return: Dicionário com o envio,resposta e reason.
        """
        p = ProcessadorNFe()
        p.versao = u'3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False
        p.contingencia = contingencia
        p.caminho = u''

        return p.inutilizar_nota(chave, justificativa)

    @staticmethod
    def inutilizar_faixa_numeracao(ano, cnpj, serie, inicio, fim, justificativa, cert, key,
                                   ambiente, estado, contingencia=False):
        """
        Realiza a inutilização de faixa de numeração de nota fiscal
        @param cnpj:cnpj do emitente
        @param serie: série da nfe
        @param inicio: faixa inicial da nota que deseja inutilizar
        @param fim: faixa final da nota que deseja inutilizar
        @param justificativa: justificativa da inutilização 
        :param cert: certificado digital A1
        :param key: chave privada do certificado digital
        :param ambiente: ambiente da consulta. 1=Producao e 2=Homologacao,
        :param estado: UF em que sera realizado o processamento
        :param contingencia: habilita o servico de contigencia
        @return: Dicionário com o envio,resposta e reason.
        """
        p = ProcessadorNFe()
        p.versao = u'3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False
        p.contingencia = contingencia
        p.caminho = u''

        return p.inutilizar_nota(ano, cnpj, serie, inicio, fim, justificativa)

    def gerar_danfe(self, xml, retcan_nfe=None, site_emitente=u'', logo=u'',
                    nome_sistema=u'SigERP - www.sigsolucoes.net.br', leiaute_logo_vertical=False):
        """
        Geração do DANFE
        @param xml:string do xml da NF-e
        @param site_emitente: Endereço do site do emitente
        @param logo:O caminho para a imagem ou  Instância da imagem.
        @param leiaute_logo_vertical: possibilita que a logomarca tenha a orientação vertical
        @return: String
        """

        d = DANFE()
        nota = NFe_310()
        protNFe = ProtNFe_310()
        proc = ProcNFe_310()
        cancNFe = RetCancNFe_310()
        nota.xml = xml
        resp = minidom.parseString(xml.encode('utf-8'))
        resp = resp.getElementsByTagName("protNFe")[0]
        resposta = resp.toxml()
        protNFe.xml = resposta
        proc.protNFe = protNFe

        # Danfe de nota fiscal cancelada
        if retcan_nfe:
            resp = minidom.parseString(retcan_nfe.encode('utf-8'))
            resp = resp.getElementsByTagName('retEvento')[0]
            resposta = resp.toxml()
            cancNFe.xml = resposta

        d.NFe = nota
        d.protNFe = protNFe
        d.retCancNFe = cancNFe
        d.versao = u'3.10'
        d.salvar_arquivo = False
        d.obs_impressao = u'DANFE gerado em %s' % self.utc_to_local(datetime.now()).strftime(
            '%d/%m/%Y as %H:%M:%S')
        d.nome_sistema = nome_sistema
        d.site = site_emitente
        d.logo = logo
        d.leiaute_logo_vertical = leiaute_logo_vertical
        d.gerar_danfe()
        danfe_pdf = StringIO()
        d.danfe.generate_by(PDFGenerator, filename=danfe_pdf)
        d.danfe = danfe_pdf.getvalue()
        danfe_pdf.close()

        return d.danfe

    @staticmethod
    def utc_to_local(utc_dt):
        local_dt = utc_dt.replace(tzinfo=pytz.utc).astimezone(local_tz)
        return local_tz.normalize(local_dt)

    @staticmethod
    def validar_chave_nfe(chave, uf, data_emissao, cnpj, modelo, serie, numero_nf, considera_cnpj):
        """
        Verifica consistência da chave de NF-e informada.

        :param chave: a chave da NF-e com 44 posicoes
        :param uf: o codigo IBGE da UF
        :param data_emissao: a data da emissao da nota fiscal
        :param cnpj: o CNPJ do emisso da NF-e
        :param modelo: o modelo da nota fiscal (normalmente 55)
        :param serie: a serie da nota fiscal
        :param numero_nf: o numero da nota fiscal
        :param considera_cnpj: Verdadeiro, sera usado para verificacao o parametro cnpj. Falso, sera
        usado para verificacao o CNPJ presente no parametro chave.
        :type chave: str
        :type uf: str
        :type data_emissao: str
        :type cnpj: str
        :type modelo: str
        :type serie: int
        :type numero_nf: int
        :type considera_cnpj: bool
        :return: informacao da verificacao da chave: sucesso (validada) ou mensagem de erro
        :rtype: dict
        """
        msg = ''
        res = {'valida': True, msg: ''}
        if len(chave) == 44:
            if chave[:2] != str(uf):
                msg += '* Estado informado inválido.\n'
            data_emissao = data_emissao[2:4] + data_emissao[5:7]
            if chave[2:6] != data_emissao:
                msg += '* Data de emissão inválida. \n'
            if not considera_cnpj:
                cnpj = chave[6:20]
            cnpj = cnpj.replace('.', '').replace('-', '').replace('/', '')
            if chave[6:20] != cnpj:
                msg += '* CNPJ do emitente inválido. \n'
            if chave[20:22] != str(modelo):
                msg += '* Modelo da Nota Fiscal inválido.\n'
            if chave[22:25] != ('%003d' % int(serie)):
                msg += '* Série da Nota Fiscal inválida.\n'
            if chave[25:34] != '%009d' % int(numero_nf):
                msg += '* Número da Nota Fiscal inválido. \n'
            nf = NFe_200()
            digito_verificador = nf._calcula_dv(chave[:43])
            if digito_verificador != int(chave[43:]):
                msg += '* Dígito verificador inválido. \n'
            if msg:
                res['msg'] = 'Chave de acesso inválida: \n' + msg
                res['valida'] = False
        else:
            res['msg'] = 'Chave inválida.'
            res['valida'] = False
        return res

    @staticmethod
    def consultar_nota(chave, cert, key, ambiente, estado, contingencia=False):
        """
        @param chave:chave da nfe
        :param cert: certificado digital A1
        :param key: chave privada do certificado digital
        :param ambiente: ambiente da consulta. 1=Producao e 2=Homologacao,
        :param estado: UF em que sera realizado o processamento
        :param contingencia: habilita o servico de contigencia
        @return: objeto processo e arquivos.
        """
        p = ProcessadorNFe()
        p.versao = u'3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.contingencia=contingencia
        p.salvar_arquivos = False

        return p.consultar_nota(chave)

    @staticmethod
    def consultar_cadastro_contribuinte(cpf_cnpj, inscricao_estadual, cert, key, ambiente, estado,
                           contingencia=False):
        """
        @param chave:chave da nfe
        :param cert: certificado digital A1
        :param key: chave privada do certificado digital
        :param ambiente: ambiente da consulta. 1=Producao e 2=Homologacao,
        :param estado: UF em que sera realizado o processamento
        :param contingencia: habilita o servico de contigencia
        @return: Dicionário com o envio,resposta e reason.
        """
        p = ProcessadorNFe()
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.contingencia = contingencia
        p.salvar_arquivos = False

        return p.consultar_cadastro_contribuinte(cpf_cnpj,  inscricao_estadual)

    @staticmethod
    def efetuar_manifesto(cnpj, evento, chave, cert, key, ambiente, estado, motivo=None):
        """
        Realiza o manifesto do destinatário
        :param cnpj: CNPJ do emitente
        :param evento: codigo do evento
        :param chave:chave da nfe
        :param cert: certificado digital A1
        :param key: chave privada do certificado digital
        :param ambiente: ambiente da consulta. 1=Producao e 2=Homologacao,
        :param estado: UF em que sera realizado o processamento
        :param motivo: Motivo utilizado na nao realizacao da operacao
        :return: Dicionario com o envio, resposta e reason.
        """
        p = ProcessadorNFe()
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False

        return p.manifesto_destinatario(cnpj, evento, chave, motivo)

    @staticmethod
    def consultar_notas_destinatario(cnpj, cert, key, nsu, ambiente, estado):
        """
        Realiza  a consulta do manifesto do destinatário
        :param cert: certificado digital A1
        :param key: chave privada do certificado digital
        :param ambiente: ambiente da consulta. 1=Producao e 2=Homologacao,
        :param estado: UF em que sera realizado o processamento
        :param contingencia: habilita o servico de contigencia
        :return: Dicionário com o envio, resposta e reason
        """
        p = ProcessadorNFe()
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False

        return p.consultar_notas_destinatario(cnpj, nsu)

    @staticmethod
    def download_xml(cnpj, chave, cert, key, ambiente, estado):
        """
        Realiza  a consulta do manifesto do destinatário
        @param chave:chave da nfe
        :param cert: certificado digital A1
        :param key: chave privada do certificado digital
        :param ambiente: ambiente da consulta. 1=Producao e 2=Homologacao,
        :param estado: UF em que sera realizado o processamento
        :param contingencia: habilita o servico de contigencia
        @return: Dicionário com o envio,resposta e reason.
        """
        p = ProcessadorNFe()
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key
        p.salvar_arquivos = False

        return p.download_xml(cnpj, chave)

    @staticmethod
    def emitir_carta_correcao(chave, cnpj, correcao, cert, key, sequencia, ambiente, estado):
        """
        @param chave:chave da nfe
        :param cert: certificado digital A1
        :param key: chave privada do certificado digital
        :param ambiente: ambiente da consulta. 1=Producao e 2=Homologacao,
        :param estado: UF em que sera realizado o processamento
        :param contingencia: habilita o servico de contigencia
        @return: Dicionário com o envio,resposta e reason.
        """
        p = ProcessadorNFe()
        p.versao = u'3.10'
        p.estado = estado
        p.ambiente = ambiente
        p.certificado.cert_str = cert
        p.certificado.key_str = key

        return p.corrigir_nota(chave, cnpj, correcao, sequencia)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
