# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
# $Id: danfe_nfce_bobina.py 531 2022-09-29 20:06:24Z felipe.franca $

from nfe.nfce.consrecinfce import ProcNFCe
from report import Line, Column, _slice
import locale
import re
from datetime import datetime

def _header(data):
    header = [Line() for l in range(8)]
    header[0].style = 'reset+bold'
    name = data.xFant.valor or data.xNome.valor
    header[0].add(Column(width=48, value=name, align='center'))
    address = '{}, {}'.format(data.enderEmit.xLgr.valor, data.enderEmit.nro.valor)
    header[1].add(Column(width=48, value=address, align='center'))
    city = '{} - {}/{}'.format(data.enderEmit.xBairro.valor, data.enderEmit.xMun.valor,
                               data.enderEmit.UF.valor)
    header[2].add(Column(width=48, value=city, align='center'))
    cnpj = data.CNPJ.valor
    cnpj = '{}.{}.{}/{}-{}'.format(cnpj[:2], cnpj[2:5], cnpj[5:8], cnpj[8:12], cnpj[-2:])
    header[3].add(Column(width=48, value='CNPJ: ' + cnpj, align='center'))
    header[4].add(Column(value='-' * 48))
    header[5].add(Column(width=48, value='DOCUMENTO AUXILIAR', align='center'))
    header[6].add(Column(width=48, value='DA NOTA FISCAL DE CONSUMIDOR ELETRONICA', align='center'))
    header[7].add(Column(value='-' * 48))

    return '\n'.join([l.content() for l in header])


def _detail(data, lang='pt_BR'):
    detail = [' # | COD | DESC | QTD | UN | VL.UNIT | VL.TOT R$', '-' * 48]
    locale.setlocale(locale.LC_ALL, str('%s.UTF-8' % lang))
    monetary2 = lambda v: locale.format('%.2f', float(v), grouping=True)
    for d in data:
        d = dict(nItem=d.nItem.valor, cEAN=d.prod.cEAN.valor, xProd1=d.prod.xProd.valor[:45],
                 qCom=d.prod.qCom.valor, uCom=d.prod.uCom.valor, vUnCom=d.prod.vUnCom.valor,
                 vProd=monetary2(d.prod.vProd.valor), xProd2=d.prod.xProd.valor[45:], filler='')
        lin1 = Line(format='{nItem:03d} {cEAN:>014} {xProd1:<45.45}', style='condensed')
        lin2 = Line(format='{filler:3.3} {xProd2:<18.18} {qCom:>17} {uCom:<.4} X {vUnCom:>8} '
                           '{vProd:>8}', style='condensed')
        for k, v in d.items():
            col = Column(title=k, value=v)
            lin1.add(col)
            lin2.add(col)
        detail.append(lin1.content())
        detail.append(lin2.content())
    detail.append('-' * 48)

    return '\n'.join(detail)


def _sumary(data):
    summary = []
    lin = Line(width=64, style='condensed')
    lin.add(Column(width=32, value='QTD. TOTAL ITENS'))
    lin.add(Column(width=32, value=len(data.det), align='right'))
    summary.append(lin.content())
    lin = Line(width=64, style='condensed')
    lin.add(Column(value='Valor total R$'))

    lin.add(Column(value=float(data.total.ICMSTot.vProd.valor), align='>', decimals=2))
    lin.autoadjust()
    summary.append(lin.content())

    lin = Line(width=64, style='condensed')
    lin.add(Column(value='Desconto R$'))
    lin.add(Column(value=float(data.total.ICMSTot.vDesc.valor), align='>', decimals=2))
    lin.autoadjust()
    summary.append(lin.content())
    lin = Line(width=64, style='condensed')
    lin.add(Column(value='Seguro R$'))
    lin.add(Column(value=float(data.total.ICMSTot.vSeg.valor), align='>', decimals=2))
    lin.autoadjust()
    summary.append(lin.content())
    lin = Line(width=64, style='condensed')
    lin.add(Column(value='Frete R$'))
    lin.add(Column(value=float(data.total.ICMSTot.vFrete.valor), align='>', decimals=2))
    lin.autoadjust()
    summary.append(lin.content())
    lin = Line(width=64, style='condensed')
    lin.add(Column(value='Outros R$'))
    lin.add(Column(value=float(data.total.ICMSTot.vOutro.valor), align='>', decimals=2))
    lin.autoadjust()
    summary.append(lin.content())

    lin = Line(width=64, style='condensed')
    lin.add(Column(value='FORMAS DE PAGAMENTO'))
    lin.add(Column(value='VALOR PAGO', align='right'))
    lin.autoadjust()
    summary.append(lin.content())

    # Formas de pagamento NFC-e
    _pay = {
        '01': u'Dinheiro',
        '02': u'Cheque',
        '03': u'Cartao de Credito',
        '04': u'Cartao de Debito',
        '05': u'Credito da Loja',
        '10': u'Vale Alimentacao',
        '11': u'Vale Refeicao',
        '12': u'Vale Presente',
        '13': u'Vale Combustivel',
        '99': u'Outras'
    }
    payment = []
    for d in data.pag.detPag:
        lin = Line(width=64, style='condensed')
        lin.add(Column(value=_pay.get(d.tPag.valor, 'Outras')))
        lin.add(Column(value=float(d.vPag.valor), align='>', decimals=2))
        lin.autoadjust()
        payment.append(lin.content())
    lin = Line(width=48)
    lin.add(Column(value='-' * 48))
    payment.append(lin.content())
    summary.extend(payment)

    return '\n'.join(summary)


def _access(data):
    access = []
    lin = Line(width=48, style='bold')
    lin.add(Column(width=48, value='Consulte pela Chave de Acesso em:', align='center'))
    access.append(lin.content())
    lin = Line(width=64, style='condensed')
    lin.add(Column(value=data.NFe.infNFeSupl.urlChave.valor, width=64, align='center'))
    access.append(lin.content())
    lin = Line(width=64, style='condensed')
    chave = data.protNFe.infProt.chNFe.valor
    lin.add(Column(value=' '.join([w for w in re.split(r'(\w{4})', chave) if w]),
                   width=64, align='center'))
    access.append(lin.content())
    lin = Line(width=48)
    lin.add(Column(value='-' * 48))
    access.append(lin.content())

    return '\n'.join(access)


def _client(data):
    client = []
    name = data.xNome.valor
    address = '{} {}, {}'.format(data.enderDest.xLgr.valor, data.enderDest.nro.valor,
                                 data.enderDest.xBairro.valor)
    city = '{}/{}'.format(data.enderDest.xMun.valor, data.enderDest.UF.valor)
    if data.CNPJ.valor:
        cnpj = data.CNPJ.valor
        cnpj = '{}.{}.{}/{}-{}'.format(cnpj[:2], cnpj[2:5], cnpj[5:8], cnpj[8:12], cnpj[-2:])
        values = 'CONSUMIDOR - CNPJ: {cnpj} - {name} - {address} - {city}'
    elif data.CPF.valor:
        cpf = data.CPF.valor
        cpf = '{}.{}.{}-{}'.format(cpf[:3], cpf[3:6], cpf[6:9], cpf[-2:])
        values = 'CONSUMIDOR - CPF: {cpf} - {name} - {address} - {city}'
    else:
        values = '{name}'
        name = 'CONSUMIDOR NAO IDENTIFICADO'
    values = values.format(**locals())

    for c in _slice(values, 64):
        lin = Line(style='condensed')
        lin.add(Column(width=64, value=c, align='center'))
        client.append(lin.content())

    return '\n'.join(client)


def _qrcode_bematech(text, width=8):
    center = '\x1b\x61\x01'  # esse codigo faz a centralizacao
    qrcode = '\x1d\x6b\x51'  # esse e o do qrcode
    width = '\x02' + chr(width) + chr(width) + '\x01'  # aqui e o tamanho do qrcode
    level_error = ''
    lenght = chr(len(text))  # tamanho do texto
    start_bit = '\x00'  # bit de partida

    return center + qrcode + width + level_error + lenght + start_bit + text


def _qrcode_elgin(text, width=4):
    center = '\x1b\x61\x31'  # esse codigo faz a centralizacao
    qrcode = '\x1d\x28\x6b\x03\x00'  # esse e o do qrcode
    width = '\x31\x43' + chr(width)
    level_error = qrcode + '\x31\x45\x32'
    length = 3 + len(text)
    pl = length & 0xff
    ph = length >> 8 & 0xff
    length = '\x1d\x28\x6b' + chr(pl) + chr(ph) + '\x31\x50\x30'
    start_bit = qrcode + '\x31\x51\x30'

    return center + qrcode + width + level_error + length + text + start_bit


def _qrcode_epson(text, width=4):
    return _qrcode_elgin(text, width)  # eh o mesmo de ElGIN


def _qrcode_daruma(text, level='Q', wmod=4):
    #  * Print QR Code
    #  * [ESC] <129> <–Size><+Size> <Width> <Ecc> <D001> <D002> . . . <Dnnn>
    #  * Size inclui os 2 bytes de controle
    #  *    Size ≤ 402
    #  *    nnn = Size – 2
    #  *   Largura do módulo (Width): 0, 4 ≤ Width ≤ 7 ( =0 para default = 5)
    #  *   Redundância (ECC): 0, M, Q, H ( =0 para cálculo automático)
    #  * Apenas para V.02.50.00 ou superior.
    #  *
    #  * @param string $data   Dados a serem inseridos no QRCode
    #  * @param string $level  Nivel de correção L,M,Q ou H
    #  * @param int    $modelo modelo de QRCode none
    #  * @param int    $wmod   largura da barra 4 ~ 7
    #  */
    # public function barcodeQRCode($data = '', $level = 'L', $modelo = 2, $wmod = 4)
    size = len(text) + 2
    if size > 402:
        return ''

    nH = int(round(size / 256, 0))
    nL = size % 256
    if wmod > 7 or wmod < 4:
        wmod = 5

    # //set error correction level
    level = level.upper()
    if level in ('M', 'Q', 'H'):
        ecc = level
    else:
        ecc = 0

    return '\x1bl\x17\x1b\x81' + chr(nL) + chr(nH) + chr(wmod) + str(ecc) + text + '\x1bl\x01'


def _identify(data, printer='bematech'):
    identify = []
    lin = Line(style='condensed')
    value = 'NFC-e No {d.NFe.infNFe.ide.nNF.valor:07} Serie {d.NFe.infNFe.ide.serie.valor:03} ' \
            '{d.NFe.infNFe.ide.dhEmi.valor:%d/%m/%Y %H:%M:%S} - Via consumidor'
    lin.add(Column(width=64, value=value.format(d=data), align='center'))
    identify.append(lin.content())
    if data.NFe.infNFe.ide.tpEmis.valor == 1:
        lin = Line(style='condensed')
        lin.add(Column(
            width=64, value='Protocolo de autorizacao: {}'.format(data.protNFe.infProt.nProt.valor),
            align='center'))
        identify.append(lin.content())
        lin = Line(style='condensed')
        lin.add(Column(width=64, value='Data de autorizacao: {:%d/%m/%Y %H:%M:%S}'.format(
            data.protNFe.infProt.dhRecbto.valor or datetime.now()), align='center'))
        identify.append(lin.content())
    else:
        lin = Line(style='bold+condensed')
        lin.add(Column(width=64, value='E M I T I D A   E M   C O N T I N G E N C I A',
                       align='center'))
        identify.append(lin.content())
        lin = Line(style='bold+condensed')
        lin.add(Column(width=64, value='P e n d e n t e   d e   a u t o r i z a c a o',
                       align='center'))
        identify.append(lin.content())

    if printer == 'bematech':
        identify.append(_qrcode_bematech(data.NFe.infNFeSupl.qrCode.valor))
    elif printer == 'elgin':
        identify.append(_qrcode_elgin(data.NFe.infNFeSupl.qrCode.valor))
    elif printer == 'epson':
        identify.append(_qrcode_epson(data.NFe.infNFeSupl.qrCode.valor))
    elif printer == 'daruma':
        identify.append(_qrcode_daruma(data.NFe.infNFeSupl.qrCode.valor))
    else:
        # Provavel...
        identify.append(_qrcode_bematech(data.NFe.infNFeSupl.qrCode.valor))

    return '\n'.join(identify)


def _complement(data):
    complement = []
    lin = Line()
    lin.add(Column(value='-' * 48))
    complement.append(lin.content())
    for c in _slice(data, 64):
        lin = Line(style='condensed')
        lin.add(Column(width=64, value=c, align='center'))
        complement.append(lin.content())
    lin = Line()
    lin.add(Column(value='-' * 48))
    complement.append(lin.content())

    return '\n'.join(complement)


def _footer():
    footer = []
    lin = Line(width=48)
    footer.append(lin.content())
    footer.append(lin.content())
    footer.append(lin.content())
    footer.append(lin.content())
    footer.append(Line(style='paper_cut').content())
    return '\n'.join(footer)


def _main():
    nfce = ProcNFCe()
    nfce.xml = open('nfce-example.xml').read()
    rep = '\x1b\x40\x1b3\x12'
    rep += '\n'.join([_header(nfce.NFe.infNFe.emit), _detail(nfce.NFe.infNFe.det),
                      _sumary(nfce.NFe.infNFe), _access(nfce), _client(nfce.NFe.infNFe.dest),
                      _identify(nfce), _complement(nfce.NFe.infNFe.infAdic.infCpl.valor),
                      _footer()])

    return rep


if __name__ == '__main__':
    report = _main()
    print (report)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
