# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2019 All Rights Reserved. Luiz Fernando Maciel Franca
              www.siginformatica.com.br <luiz@siginformatica.com.br>
                                -------

Copyleft (CC) 2019, Creative Commons
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""
import hashlib


def qrcode(ambiente, url, chave, digval, csc, cpfcnpj=None, emissao=None, valor=0, vr_icms=0):
    """
    Constroi texto do QR-Code da nota fiscal de consumidor. (online)

    :param ambiente: o ambiente utilizado na NFC-e: 1=PRODUCAO e 2=HOMOLOGACAO
    :param url: a url utilizada na chamada de validacao do QR-Code
    :param chave: a chave da NFC-e (44 caracteres)
    :param digval: valor DigestValue da NFC-e assinada
    :param csc: lista com o ID e o numero do CSC
    :param cpfcnpj: CPF ou CNPJ do destinatario da nota fiscal
    :param emissao: data da emissao da NFC-e (formato UTC - 2018-09-27T16:20:34-03:00)
    :param valor: valor da nota fiscal
    :param vr_icms: valor total do ICMS tributado
    :type ambiente: int
    :type url: str
    :type chave: str
    :type digval: str
    :type csc: list
    :type emissao: str
    :type valor: float
    :type vr_icms: float
    :return: texto com o QR-Code a ser impresso no DANFE NFC-e.
    :rtype: str
    """
    csc_id, csc = csc[0], csc[1]
    digval_hex = hashlib.sha1(digval).hexdigest()
    emissao = emissao.encode('hex')
    url1 = u'<![CDATA[%s]]>'
    url2 = u'chNFe=%(chave)s&nVersao=100&tpAmb=%(ambiente)s&cDest=%(cpfcnpj)s&dhEmi=%(emissao)s' \
           u'&vNF=%(valor).2f&vICMS=%(vr_icms).2f&digVal=%(digval_hex)s&cIdToken=%(csc_id)06d'
    url2 = url2.replace('&cDest=%(reg_id)s', '') if not cpfcnpj else url2
    url2 = url2 % locals()
    url2 += csc  # Acrescenta CSC na url antes de gerar o HASH
    hash_url = hashlib.sha1(url2).hexdigest()
    url2 = url2[:-len(csc)] + hash_url
    result = url1 % (url + url2)

    return result


def qrcode_v2_online(ambiente, url, chave, csc):
    """
    Constroi texto do QR-Code versao 2.0 da nota fiscal de consumidor. (online)

    :param ambiente: o ambiente utilizado na NFC-e: 1=PRODUCAO e 2=HOMOLOGACAO
    :param url: a url utilizada na chamada de validacao do QR-Code
    :param chave: a chave da NFC-e (44 caracteres)
    :param csc: lista com o ID e o numero do CSC
    :type ambiente: int
    :type url: str
    :type chave: str
    :type csc: list
    :return: texto do QR-Code
    :rtype: str
    """
    url1 = u'%s?p=' % url
    url2 = u'%s|%s|%s|%s%s' % (chave, 2, ambiente, csc[0], csc[1].strip())
    hash_url = hashlib.sha1(url2).hexdigest()
    result = '%s%s|%s' % (url1, url2[:-len(csc[1].strip())], hash_url)  # URL vai sem o CSC!

    return result


def qrcode_v2_offline(ambiente, url, chave, digval, csc, emissao=None, valor=0):
    """
    Constroi texto do QR-Code versao 2.0 da nota fiscal de consumidor. (Contingencia offline)

    :param ambiente: o ambiente utilizado na NFC-e: 1=PRODUCAO e 2=HOMOLOGACAO
    :param url: a url utilizada na chamada de validacao do QR-Code
    :param chave: a chave da NFC-e (44 caracteres)
    :param digval: valor DigestValue da NFC-e assinada
    :param csc: lista com o ID e o numero do CSC
    :param emissao: data da emissao da NFC-e (DD-MM-AAAA)
    :param valor: valor da nota fiscal
    :type ambiente: int
    :type url: str
    :type chave: str
    :type digval: str
    :type csc: list
    :type emissao: str
    :type valor: float
    :return: texto do QR-Code
    :rtype: str
    """
    url1 = u'%s?p=' % url
    digval_hex = digval.encode('hex')
    url2 = u'%s|%s|%s|%s|%s|%s|%s%s' % (
        chave, 2, ambiente, '%02d' % emissao, '%.2f' % valor, digval_hex, csc[0], csc[1].strip())
    hash_url = hashlib.sha1(url2).hexdigest()
    result = '%s%s|%s' % (url1, url2[:-len(csc[1].strip())], hash_url)  # URL vai sem o CSC!

    return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
