from HTMLParser import HTMLParser

_slice = lambda l, s=48: [l[r:r + s] for r in range(0, len(l) + 1, s)]
ON = 1
OFF = 0


class ElementReport(object):
    """Base element report"""
    visible = True
    value = None
    width = 0

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class Column(ElementReport):
    """Column"""
    title = None
    format = None
    align = '<'  # Left (default)
    float = False
    truncate = False
    decimals = False
    fill = ' '

    def __init__(self, **kwargs):
        super(Column, self).__init__(**kwargs)

    def __str__(self):
        return self.content()

    def content(self):
        value = str(self.value) if isinstance(self.value, (int, long)) else self.value or ''
        try:
            value = HTMLParser().unescape(value)
        except:
            pass
        width = len(str(value)) if self.width < len(str(value)) else self.width
        decimals = isinstance(self.decimals, int) and self.decimals or 2
        if isinstance(value, float):
            value = self.decimals and '%{prec}.{dec}f'.format(prec=self.width, dec=decimals) % value
            value = ','.join(value.rsplit('.', 1))
        value = value or self.value or ''
        if self.truncate:
            truncate = '.%d' % (self.width or width)
            width = self.width or len(str(self.value))
        else:
            truncate = '.%d' % len(str(value))
        align = self.align in '<^>' and self.align
        align = not align and {'right': '>', 'center': '^'}.setdefault(self.align, '<') or align
        fill = self.fill or ' '
        if not self.format:
            _content = '{:{fill}{align}{width}{truncate}}'.format(
                value, align=align, width=width, fill=fill, truncate=truncate)
        else:
            _content = self.format.format(value)

        return self.visible and _content or ''


class Style(object):
    def __init__(self, **kwargs):
        # Default Bematech
        # style = <ctrl-OFF>, <ctrl-ON>
        self.reset = None, '\x1b\x40'
        self.bold = '\x1bF', '\x1bE'
        self.underline = '\x1b-0', '\x1b-1'
        self.expanded = '\x1b\x570', '\x1b\x571'
        self.condensed = '\x1bP', '\x1b\x0f',
        self.paper_cut = None, '\x1bm'
        super(Style, self).__init__(**kwargs)
        for k, v in kwargs.items():
            setattr(self, k, v or '')


class Line(ElementReport):
    """Line"""
    delimiter = ''
    max_num_col = False
    auto_adjust = False
    _label = False
    _columns = []
    _styles = Style()  # Bematech

    def __init__(self, **kwargs):
        self._columns = []
        self.label = False
        self.format = None
        self.style = None
        super(Line, self).__init__(**kwargs)

    def __str__(self):
        return self.content()

    def add(self, column, priority=0):
        if isinstance(column, Column):
            if priority:
                self._columns.insert(priority - 1, column)
            else:
                self._columns.append(column)

    def remove(self, column):
        if isinstance(column, str):
            # Remove by title
            pos = column in [c.title for c in self._columns]
        elif isinstance(column, Column):
            # Remove by column
            pos = self._columns.index(column)
        elif isinstance(column, (int, long)):
            # Remove by position
            pos = column
        else:
            pos = None
        if -1 < pos < self._columns.__len__() or pos in self._columns:
            self._columns.pop(pos)

    def autoadjust(self):
        prop1 = self.width / len(self._columns)
        prop2 = sum([col.width for col in self._columns]) + len(self._columns) + 1
        prop2 /= len(self._columns)
        for c in self._columns:
            c.truncate = True
            c.width = prop1 if prop1 > prop2 else prop2

    def content(self):
        _lin = self.delimiter.join([
            col.content() if col.visible else ' ' * (col.width or len(col.value))
            for col in self._columns])
        _lin = self.delimiter + _lin + self.delimiter
        if self.format:
            _lin = self.format.format(**dict([(c.title, c.value) for c in self._columns]))
        styles = self.style and '+' in self.style and [s.strip() for s in self.style.split('+')]
        styles = not styles and [self.style] or styles
        styles_on = ''.join([getattr(self._styles, s)[ON] for s in styles if s])
        styles_off = ''.join([getattr(self._styles, s)[OFF] or '' for s in styles if s])
        return styles_on + _lin + styles_off

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
